/*
    Парус 8 - Панели мониторинга - ТОиР - Прогнозирование технического состояния
    Закладка прогнозирования
*/

//---------------------
//Подключение библиотек
//---------------------

import React, { useState, useContext } from "react"; //Классы React
import { Box, Grid } from "@mui/material"; //Интерфейсные компоненты
import { RichTreeView } from "@mui/x-tree-view/RichTreeView"; //Дерево
import { MessagingСtx } from "../../context/messaging"; //Контекст сообщений
import { P8PDataGrid, P8P_DATA_GRID_SIZE } from "../../components/p8p_data_grid"; //Таблица данных
import { P8P_DATA_GRID_CONFIG_PROPS } from "../../config_wrapper"; //Подключение компонентов к настройкам приложения
import { useEqConfigTree, useEqConfigTechObjTable, useEqConfigTechObjCard, findTreeItem, needLoadLevel } from "./forecast_tab_hooks"; //Вспомогательные хуки
import { TechObjCard, eqConfigTechObjTableValueFormatter } from "./forecast_tab_layout"; //Вспомогательные компоненты и вёрстка

//---------
//Константы
//---------

//Высота главного меню
const APP_BAR_HEIGHT = "64px";

//Высота закладок
const TABS_HEIGHT = "49px";

//Высота кнопки "Ещё"
const TABLE_MORE_HEIGHT = "49px";

//Высота фильтров таблицы
const TABLE_FILTERS_HEIGHT = "48px";

//Стили
const STYLES = {
    LEFT_SIDE_GRID: {},
    RIGHT_SIDE_GRID: {},
    TREE_BOX: {
        minHeight: `calc(100vh - ${APP_BAR_HEIGHT} - ${TABS_HEIGHT})`,
        maxHeight: `calc(100vh - ${APP_BAR_HEIGHT} - ${TABS_HEIGHT})`,
        overflowX: "hidden",
        overflowY: "scroll",
        scrollbarWidth: "thin",
        padding: "3px"
    },
    TECH_OBJ_TABLE: (morePages, filters) => ({
        height: `calc(100vh - ${APP_BAR_HEIGHT} - ${TABS_HEIGHT} - ${morePages ? TABLE_MORE_HEIGHT : "0px"} - ${
            filters ? TABLE_FILTERS_HEIGHT : "0px"
        })`,
        scrollbarWidth: "thin"
    })
};

//Начальное состояние спецификации техничких объектов
const TECH_OBJ_SPEC_INIT = { parent: null, filters: [], orders: [], pageNumber: 1 };

//-----------
//Тело модуля
//-----------

//Закладка прогнозирования
const ForecastTab = () => {
    //Подключение к контексту сообщений
    const { InlineMsgInfo } = useContext(MessagingСtx);

    //Собственное состояние - текущая загружаемая ветка
    const [loadingTreeItem, setLoadingTreeItem] = useState(0);

    //Собственное состояние - спецификация технических объектов
    const [techObjSpec, setTechObjSpec] = useState({ ...TECH_OBJ_SPEC_INIT });

    //Собственное состояния - карточка технического объекта
    const [techObjCardId, setTechObjCardId] = useState(null);

    //Загрузчик веток дерева
    const { tree } = useEqConfigTree(loadingTreeItem);

    //Загрузчик технических объектов для выбранной ветки
    const { techObjsDataGrid, techObjsDataGridIsLoading } = useEqConfigTechObjTable(
        techObjSpec.parent,
        techObjSpec.filters,
        techObjSpec.orders,
        techObjSpec.pageNumber
    );

    //Загрузчик карточки выбоанного технического объекта
    const { techObjCard, techObjCardIsLoading } = useEqConfigTechObjCard(techObjCardId);

    //Обработка развёртывания/свёртывания уровня дерева
    const handleTreeItemExpansionToggle = (event, itemId, isExpanded) =>
        isExpanded && needLoadLevel(tree, itemId) ? setLoadingTreeItem(parseInt(itemId)) : null;

    //Обработка фокусировки на элементе дерева
    const handleTreeItemFocus = (event, itemId) => {
        const item = findTreeItem(tree, itemId);
        if (item && item?.showCard) setTechObjCardId(parseInt(itemId));
        else {
            setTechObjCardId(null);
            setTechObjSpec({ ...TECH_OBJ_SPEC_INIT, parent: parseInt(itemId) });
        }
    };

    //При изменении состояния фильтров спецификации технических объектов
    const handleTechObjSpecFilterChanged = ({ filters }) => setTechObjSpec(pv => ({ ...pv, filters: [...filters], pageNumber: 1 }));

    //При изменении состояния сортировки спецификации технических объектов
    const handleTechObjSpecOrderChanged = ({ orders }) => setTechObjSpec(pv => ({ ...pv, orders: [...orders], pageNumber: 1 }));

    //При изменении количества отображаемых страниц спецификации технических объектов
    const handleTechObjSpecPagesCountChanged = () => setTechObjSpec(pv => ({ ...pv, pageNumber: pv.pageNumber + 1 }));

    //Текст при отсутствии данных в списке технических объектов
    const noDataFoundText =
        techObjsDataGridIsLoading || !techObjsDataGrid.init
            ? "Минуточку..."
            : techObjSpec.filters.length > 0
            ? "Нет данных, соответствующих условиям отбора"
            : "Выбранный уровень структуры не содержит данных о технических объектах";

    //Генерация содержимого
    return (
        <div>
            <Grid container>
                <Grid item xs={3} sx={STYLES.LEFT_SIDE_GRID}>
                    <Box sx={STYLES.TREE_BOX}>
                        <RichTreeView items={tree} onItemExpansionToggle={handleTreeItemExpansionToggle} onItemFocus={handleTreeItemFocus} />
                    </Box>
                </Grid>
                <Grid item xs={9} sx={STYLES.RIGHT_SIDE_GRID}>
                    {techObjCardId ? (
                        !techObjCardIsLoading ? (
                            <TechObjCard cardData={techObjCard} />
                        ) : null
                    ) : techObjsDataGrid.init ? (
                        <P8PDataGrid
                            {...{ ...P8P_DATA_GRID_CONFIG_PROPS, noDataFoundText }}
                            containerComponentProps={{
                                sx: STYLES.TECH_OBJ_TABLE(techObjsDataGrid.morePages, techObjSpec.filters.length > 0),
                                elevation: 0
                            }}
                            filtersInitial={techObjSpec.filters}
                            columnsDef={techObjsDataGrid.columnsDef}
                            rows={techObjsDataGrid.rows}
                            size={P8P_DATA_GRID_SIZE.SMALL}
                            morePages={techObjsDataGrid.morePages}
                            fixedHeader={true}
                            reloading={false}
                            valueFormatter={eqConfigTechObjTableValueFormatter}
                            onFilterChanged={handleTechObjSpecFilterChanged}
                            onOrderChanged={handleTechObjSpecOrderChanged}
                            onPagesCountChanged={handleTechObjSpecPagesCountChanged}
                        />
                    ) : (
                        <InlineMsgInfo okBtn={false} text={"Укажите элемент структуры состава оборудования для отображения данных"} />
                    )}
                </Grid>
            </Grid>
        </div>
    );
};

//----------------
//Интерфейс модуля
//----------------

export { ForecastTab };
